Package: rdmulti
Type: Package
Title: Robust Local Polynomial Methods for RD Designs with Multiple
        Cutoffs or Multiple Scores
Version: 2.0.0
Date: 2026-05-15
Authors@R: c(person(given = c("Matias", "D."),
                      family = "Cattaneo",
                      role = c("aut", "cre"),
                      email = "matias.d.cattaneo@gmail.com"),
               person(given = "Rocio",
                      family = "Titiunik",
                      role = "aut",
                      email = "rocio.titiunik@gmail.com"),
               person(given = "Gonzalo",
                      family = "Vazquez-Bare",
                      role = "aut",
                      email = "gvazquezbare@gmail.com"))
Description: The 'rdmulti' package implements estimation, inference, and graphical procedures for regression discontinuity (RD) designs with multiple cutoffs or multiple scores. rdmc() provides point estimation and robust bias-corrected inference for multi-cutoff designs, rdmcplot() provides data-driven RD plots for multi-cutoff designs, and rdms() provides point estimation and robust bias-corrected inference for multi-score designs. See Cattaneo, Titiunik and Vazquez-Bare (2020) <https://rdpackages.github.io/references/Cattaneo-Titiunik-VazquezBare_2020_Stata.pdf> for further methodological details.
Imports: ggplot2, rdrobust, rlang
License: GPL-3
Encoding: UTF-8
URL: https://github.com/rdpackages/rdmulti
BugReports: https://github.com/rdpackages/rdmulti/issues
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-16 21:40:59 UTC; CodexSandboxOffline
Author: Matias D. Cattaneo [aut, cre],
  Rocio Titiunik [aut],
  Gonzalo Vazquez-Bare [aut]
Maintainer: Matias D. Cattaneo <matias.d.cattaneo@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-17 22:40:07 UTC
Built: R 4.6.0; ; 2026-05-18 00:56:37 UTC; unix

# Generated by roxygen2: do not edit by hand

export(rdmc)
export(rdmcplot)
export(rdms)
import(ggplot2)
import(rdrobust)
importFrom(grDevices,rgb)
importFrom(graphics,abline)
importFrom(graphics,arrows)
importFrom(graphics,barplot)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(rlang,.data)
importFrom(stats,na.omit)
importFrom(stats,pnorm)
importFrom(stats,poly)
importFrom(stats,qnorm)

###################################################################
# rdms: point estimation and robust bias-corrected inference for multi-score designs
# !version 2.0.0 15-May-2026
# Authors: Matias Cattaneo, Rocio Titiunik, Gonzalo Vazquez-Bare
###################################################################

#' Point estimation and robust bias-corrected inference for multi-score designs
#'
#' \code{rdms()} implements point estimation and robust bias-corrected inference
#' for Regression Discontinuity (RD) designs with multiple scores, including
#' cumulative-cutoff designs and designs with two running variables.
#'
#'
#' @author
#' Matias D. Cattaneo, Princeton University. \email{matias.d.cattaneo@gmail.com}
#'
#' Rocio Titiunik, Princeton University. \email{rocio.titiunik@gmail.com}
#'
#' Gonzalo Vazquez-Bare, UC Santa Barbara. \email{gvazquezbare@gmail.com}
#'
#' @references
#'
#' Cattaneo, M.D., R. Titiunik and G. Vazquez-Bare. (2020). \href{https://rdpackages.github.io/references/Cattaneo-Titiunik-VazquezBare_2020_Stata.pdf}{Analysis of Regression Discontinuity Designs with Multiple Cutoffs or Multiple Scores}. \emph{Stata Journal}, forthcoming.
#'
#'
#' @param Y outcome variable.
#' @param X running variable.
#' @param C vector of cutoffs.
#' @param X2 if specified, second running variable.
#' @param zvar if X2 is specified, treatment indicator.
#' @param C2 if specified, second vector of cutoffs.
#' @param rangemat matrix of cutoff-specific ranges for the running variable.
#' @param xnorm normalized running variable to estimate pooled effect.
#' @param fuzzy specifies a fuzzy design. See \code{rdrobust()} for details.
#' @param derivvec vector of cutoff-specific order of derivatives. See
#'   \code{rdrobust()} for details.
#' @param pooled_opt options to be passed to \code{rdrobust()} to calculate
#'   pooled estimand.
#' @param pvec vector of cutoff-specific polynomial orders. See
#'   \code{rdrobust()} for details.
#' @param qvec vector of cutoff-specific polynomial orders for bias estimation.
#'   See \code{rdrobust()} for details.
#' @param hmat matrix of cutoff-specific bandwidths. See \code{rdrobust()} for
#'   details.
#' @param bmat matrix of cutoff-specific bandwidths for bias estimation. See
#'   \code{rdrobust()} for details.
#' @param rhovec vector of cutoff-specific values of rho. See \code{rdrobust()}
#'   for details.
#' @param covs_mat matrix of covariates. See \code{rdplot()} for details.
#' @param covs_list list of of covariates to be used in each cutoff.
#' @param covs_dropvec vector indicating whether collinear covariates should be
#'   dropped at each cutoff. See \code{rdrobust()} for details.
#' @param kernelvec vector of cutoff-specific kernels. See \code{rdrobust()} for
#'   details.
#' @param weightsvec vector of length equal to the number of cutoffs indicating
#'   the names of the variables to be used as weights in each cutoff. See \code{rdrobust()} for details.
#' @param bwselectvec vector of cutoff-specific bandwidth selection methods. See
#'   \code{rdrobust()} for details.
#' @param scaleparvec vector of cutoff-specific scale parameters. See
#'   \code{rdrobust()} for details.
#' @param scaleregulvec vector of cutoff-specific scale regularization
#'   parameters. See \code{rdrobust()} for details.
#' @param masspointsvec vector indicating how to handle repeated values at each
#'   cutoff. See \code{rdrobust()} for details.
#' @param bwcheckvec vector indicating the value of bwcheck at each cutoff. See
#'   \code{rdrobust()} for details.
#' @param bwrestrictvec vector indicating whether computed bandwidths are
#'   restricted to the range or runvar at each cutoff. See \code{rdrobust()} for
#'   details.
#' @param stdvarsvec vector indicating whether variables are standardized at
#'   each cutoff. See \code{rdrobust()} for details.
#' @param vcevec vector of cutoff-specific variance-covariance estimation
#'   methods. See \code{rdrobust()} for details.
#' @param nnmatchvec vector of cutoff-specific nearest neighbors for variance
#'   estimation. See \code{rdrobust()} for details.
#' @param cluster cluster ID variable. See \code{rdrobust()} for details.
#' @param ginv.tolvec vector of cutoff-specific tolerances for generalized
#'   inverse calculations. See \code{rdrobust()} for details.
#' @param sharpbwvec vector indicating whether fuzzy RD bandwidth selection
#'   uses the sharp RD model at each cutoff. See \code{rdrobust()} for details.
#' @param level confidence level for confidence intervals. See \code{rdrobust()}
#'   for details.
#' @param plot plots cutoff-specific and pooled estimates.
#' @param conventional reports conventional, instead of robust-bias corrected,
#'   p-values and confidence intervals.
#' @param subset optional subset of observations to use.
#' @param data optional data frame for resolving string variable names.
#'
#'
#' @return \item{B}{vector of bias-corrected coefficients}
#' \item{V}{variance-covariance matrix of the estimators} \item{Coefs}{vector of
#' conventional coefficients} \item{Nh}{vector of sample sizes within bandwidth
#' at each cutoff} \item{CI}{bias corrected confidence intervals}
#' \item{H}{bandwidth used at each cutoff} \item{Pv}{vector of robust p-values}
#'
#'
#' @examples
#' # Toy dataset: cumulative cutoffs
#' X <- runif(1000,0,100)
#' C <- c(33,66)
#' Y <- (1+X)*(X<C[1])+(0.8+0.8*X)*(X>=C[1]&X<C[2])+(1.2+1.2*X)*(X>=C[2]) + rnorm(1000)
#' # rmds: basic syntax
#' tmp <- rdms(Y,X,C)
#'
#'
#' @export

rdms <- function(Y,X,C,X2=NULL,zvar=NULL,C2=NULL,rangemat=NULL,xnorm=NULL,
                 fuzzy=NULL,derivvec=NULL,pooled_opt=NULL,pvec=NULL,qvec=NULL,
                 hmat=NULL,bmat=NULL,rhovec=NULL,covs_mat=NULL,covs_list=NULL,
                 covs_dropvec=NULL,kernelvec=NULL,weightsvec=NULL,
                 bwselectvec=NULL,scaleparvec=NULL,scaleregulvec=NULL,
                 masspointsvec=NULL,bwcheckvec=NULL,bwrestrictvec=NULL,
                 stdvarsvec=NULL,vcevec=NULL,nnmatchvec=NULL,cluster=NULL,
                 ginv.tolvec=NULL,sharpbwvec=NULL,level=95,plot=FALSE,
                 conventional=FALSE,subset=NULL,data=NULL){

  #################################################################
  # Setup and error checking
  #################################################################

  if (!is.null(data)){
    if (is.character(Y) && length(Y)==1) Y <- data[[Y]]
    if (is.character(X) && length(X)==1) X <- data[[X]]
    if (is.character(C) && length(C)==1) C <- unique(na.omit(data[[C]]))
    if (!is.null(X2) && is.character(X2) && length(X2)==1) X2 <- data[[X2]]
    if (!is.null(zvar) && is.character(zvar) && length(zvar)==1) zvar <- data[[zvar]]
    if (!is.null(C2) && is.character(C2) && length(C2)==1) C2 <- unique(na.omit(data[[C2]]))
    if (!is.null(xnorm) && is.character(xnorm) && length(xnorm)==1) xnorm <- data[[xnorm]]
    if (!is.null(fuzzy) && is.character(fuzzy) && length(fuzzy)==1) fuzzy <- data[[fuzzy]]
    if (!is.null(cluster) && is.character(cluster) && length(cluster)==1) cluster <- data[[cluster]]
    if (!is.null(covs_mat) && is.character(covs_mat)) covs_mat <- data[,covs_mat,drop=FALSE]
  }

  n.orig <- length(Y)
  if (!is.null(subset)){
    Y <- Y[subset]
    X <- X[subset]
    if (!is.null(X2) && length(X2)==n.orig) X2 <- X2[subset]
    if (!is.null(zvar) && length(zvar)==n.orig) zvar <- zvar[subset]
    if (!is.null(xnorm) && length(xnorm)==n.orig) xnorm <- xnorm[subset]
    if (!is.null(fuzzy) && length(fuzzy)==n.orig) fuzzy <- fuzzy[subset]
    if (!is.null(cluster) && length(cluster)==n.orig) cluster <- cluster[subset]
    if (!is.null(covs_mat) && nrow(as.matrix(covs_mat))==n.orig) covs_mat <- as.matrix(covs_mat)[subset,,drop=FALSE]
  }

  if (!is.null(X2) & is.null(zvar)) stop('Need to specify zvar when X2 is specified')
  if (!is.null(X2) & is.null(C2)) stop('Need to specify C2 if X2 is specified')

  cnum <- length(C)

  if (!is.null(C2)){
    if (cnum!=length(C2)) stop('cutoff coordinates incorrectly specified')
  }

  if (!is.null(rangemat)){
    if (is.null(dim(rangemat))){
      rangemat <- matrix(rangemat,nrow=cnum,ncol=2)
    }
  }  else {
    rangemat <- cbind(rep(-Inf,cnum),rep(Inf,cnum))
  }

  if (!is.null(hmat)){
    if (is.null(dim(hmat))){
      hmat <- matrix(hmat,nrow=cnum,ncol=2)
    }
  }
  if (!is.null(bmat)){
    if (is.null(dim(bmat))){
      bmat <- matrix(bmat,nrow=cnum,ncol=2)
    }
  }

  if (is.null(covs_dropvec)) covs_dropvec <- rep(TRUE,cnum)
  if (is.null(kernelvec)) kernelvec <- rep('tri',cnum)
  if (is.null(bwselectvec)) bwselectvec <- rep('mserd',cnum)
  if (is.null(vcevec)) vcevec <- rep('nn',cnum)
  if (is.null(nnmatchvec)) nnmatchvec <- rep(3,cnum)
  if (is.null(scaleparvec)) scaleparvec <- rep(1,cnum)
  if (is.null(scaleregulvec)) scaleregulvec <- rep(1,cnum)
  if (is.null(masspointsvec)) masspointsvec <- rep('adjust',cnum)
  if (is.null(bwrestrictvec)) bwrestrictvec <- rep(TRUE,cnum)
  if (is.null(stdvarsvec)) stdvarsvec <- rep(FALSE,cnum)
  if (is.null(ginv.tolvec)) ginv.tolvec <- rep(1e-20,cnum)
  if (is.null(sharpbwvec)) sharpbwvec <- rep(FALSE,cnum)
  if (!is.null(covs_mat)){
    covs_mat <- as.matrix(covs_mat)
    if (!is.null(covs_list)){
      if (length(covs_list)!=cnum) stop('Elements in covs_list should equal number of cutoffs')
    }
  }

  B <- matrix(NA,nrow=1,ncol=cnum+1)
  V <- matrix(NA,nrow=1,ncol=cnum+1)
  Coefs <- matrix(NA,nrow=1,ncol=cnum+1)
  V_cl <- matrix(NA,nrow=1,ncol=cnum+1)
  Nh <- matrix(NA,nrow=2,ncol=cnum+1)
  CI <- matrix(NA,nrow=2,ncol=cnum+1)
  CI_cl <- matrix(NA,nrow=2,ncol=cnum+1)
  Pv <- matrix(NA,nrow=1,ncol=cnum+1)
  Pv_cl <- matrix(NA,nrow=1,ncol=cnum+1)
  H <- matrix(NA,nrow=2,ncol=cnum+1)
  Bbw <- matrix(NA,nrow=2,ncol=cnum+1)

  c.disp <- NULL

  #################################################################
  # Calculate cutoff-specific estimates
  #################################################################

  if (is.null(X2)){

    for (c in 1:cnum){

      xc.full <- X - C[c]
      Rc <- rangemat - C

      use <- xc.full>=Rc[c,1] & xc.full<=Rc[c,2]
      yc <- Y[use]
      xc <- xc.full[use]

      if (!is.null(cluster)){
        cc <- cluster[use]
      } else {
        cc <- NULL
      }

      if (!is.null(weightsvec)){
        weightaux <- weightsvec[c]
        weightsc <- eval(parse(text=weightaux), parent.frame())[use]
      } else{
        weightsc <- NULL
      }

      if (!is.null(covs_mat)){
        covs_mat_c <- covs_mat[use,,drop=FALSE]
        if (!is.null(covs_list)){
          covs_aux <- covs_mat_c[,covs_list[[c]]]
        } else{
          covs_aux <- covs_mat_c
        }
      } else {
        covs_aux <- NULL
      }
      if (!is.null(fuzzy)){
        fuzzyc <- fuzzy[use]
      } else {
        fuzzyc <- NULL
      }

      rdr.tmp <- rdrobust::rdrobust(yc,xc,
                                   fuzzy=fuzzyc,
                                   deriv=derivvec[c],
                                   p=pvec[c],
                                   q=qvec[c],
                                   h=hmat[c,],
                                   b=bmat[c,],
                                   rho=rhovec[c],
                                   covs=covs_aux,
                                   covs_drop=covs_dropvec[c],
                                   ginv.tol=ginv.tolvec[c],
                                   kernel=kernelvec[c],
                                   weights=weightsc,
                                   bwselect=bwselectvec[c],
                                   scalepar=scaleparvec[c],
                                   scaleregul=scaleregulvec[c],
                                   masspoints=masspointsvec[c],
                                   bwcheck=bwcheckvec[c],
                                   bwrestrict=bwrestrictvec[c],
                                   stdvars=stdvarsvec[c],
                                   vce=vcevec[c],
                                   nnmatch=nnmatchvec[c],
                                   cluster=cc,
                                   sharpbw=sharpbwvec[c],
                                   level=level)

      B[1,c] <- rdr.tmp$Estimate[2]
      V[1,c] <- rdr.tmp$se[3]^2
      Coefs[1,c] <- rdr.tmp$Estimate[1]
      V_cl[1,c] <- rdr.tmp$se[1]^2
      CI[,c] <- rdr.tmp$ci[3,]
      CI_cl[,c] <- rdr.tmp$ci[1,]
      H[,c] <- rdr.tmp$bws[1,]
      Bbw[,c] <- rdr.tmp$bws[2,]
      Nh[,c] <- rdr.tmp$N_h
      Pv[1,c] <- rdr.tmp$pv[3]
      Pv_cl[1,c] <- rdr.tmp$pv[1]

      c.disp <- c(c.disp,round(C[c],2))

    }

  } else {

    for (c in 1:cnum){

      xc.full <- sqrt((X-C[c])^2+(X2-C2[c])^2)*(2*zvar-1)

      use <- xc.full>=rangemat[c,1] & xc.full<=rangemat[c,2]
      yc <- Y[use]
      xc <- xc.full[use]

      if (!is.null(cluster)){
        cc <- cluster[use]
      } else {
        cc <- NULL
      }

      if (!is.null(weightsvec)){
        weightaux <- weightsvec[c]
        weightsc <- eval(parse(text=weightaux), parent.frame())[use]
      } else{
        weightsc = NULL
      }

      if (!is.null(covs_mat)){
        covs_mat_c <- covs_mat[use,,drop=FALSE]
        if (!is.null(covs_list)){
          covs_aux <- covs_mat_c[,covs_list[[c]]]
        } else{
          covs_aux <- covs_mat_c
        }
      } else {
        covs_aux <- NULL
      }
      if (!is.null(fuzzy)){
        fuzzyc <- fuzzy[use]
      } else {
        fuzzyc <- NULL
      }

      rdr.tmp <- rdrobust::rdrobust(yc,xc,
                                   fuzzy=fuzzyc,
                                   deriv=derivvec[c],
                                   p=pvec[c],
                                   q=qvec[c],
                                   h=hmat[c,],
                                   b=bmat[c,],
                                   rho=rhovec[c],
                                   covs=covs_aux,
                                   covs_drop=covs_dropvec[c],
                                   ginv.tol=ginv.tolvec[c],
                                   kernel=kernelvec[c],
                                   weights=weightsc,
                                   bwselect=bwselectvec[c],
                                   scalepar=scaleparvec[c],
                                   scaleregul=scaleregulvec[c],
                                   masspoints=masspointsvec[c],
                                   bwcheck=bwcheckvec[c],
                                   bwrestrict=bwrestrictvec[c],
                                   stdvars=stdvarsvec[c],
                                   vce=vcevec[c],
                                   nnmatch=nnmatchvec[c],
                                   cluster=cc,
                                   sharpbw=sharpbwvec[c],
                                   level=level)

      B[1,c] <- rdr.tmp$Estimate[2]
      V[1,c] <- rdr.tmp$se[3]^2
      Coefs[1,c] <- rdr.tmp$Estimate[1]
      V_cl[1,c] <- rdr.tmp$se[1]^2
      CI[,c] <- rdr.tmp$ci[3,]
      CI_cl[,c] <- rdr.tmp$ci[1,]
      H[,c] <- rdr.tmp$bws[1,]
      Bbw[,c] <- rdr.tmp$bws[2,]
      Nh[,c] <- rdr.tmp$N_h
      Pv[1,c] <- rdr.tmp$pv[3]
      Pv_cl[1,c] <- rdr.tmp$pv[1]

      c.disp <- c(c.disp,paste0('(',round(C[c],2),',',round(C2[c],2),')'))

    }

  }


  #################################################################
  # Calculate pooled estimates
  #################################################################

  if (!is.null(xnorm)){

    aux1 <- paste0('rdrobust::rdrobust(Y,xnorm,fuzzy=fuzzy,',pooled_opt,')')

    rdr <- eval(parse(text=aux1))

    B[1,cnum+1] <- rdr$Estimate[2]
    V[1,cnum+1] <- rdr$se[3]^2
    Coefs[1,cnum+1] <- rdr$Estimate[1]
    V_cl[1,cnum+1] <- rdr$se[1]^2
    CI[,cnum+1] <- rdr$ci[3,]
    CI_cl[,cnum+1] <- rdr$ci[1,]
    H[,cnum+1] <- rdr$bws[1,]
    Bbw[,cnum+1] <- rdr$bws[2,]
    Nh[,cnum+1] <- rdr$N_h
    Pv[1,cnum+1] <- rdr$pv[3]
    Pv_cl[1,cnum+1] <- rdr$pv[1]

  }


  #################################################################
  # Display results
  #################################################################

  cat('\n')
  cat(paste0(rep('=',80),collapse='')); cat('\n')
  cat(format('Cutoff',             width=17))
  cat(format('Coef.',              width=9))
  cat(format('P-value',            width=17))
  cat(format(paste0(level,'% CI'), width=16))
  cat(format('hl',                 width=9))
  cat(format('hr',                 width=10))
  cat(format('Nh',                 width=10)); cat('\n')
  cat(paste0(rep('=',80),collapse='')); cat('\n')

  if(conventional==FALSE){
    for (k in 1:cnum){
      if (is.null(C2)){
        cat(format(sprintf('%4.2f',c.disp[k]),       width=17))
      } else {
        cat(paste0('(',format(sprintf('%4.2f',C[k])),',',format(sprintf('%4.2f',C2[k])),format(')',width=5)))
      }
      cat(format(sprintf('%7.3f',Coefs[k]),        width=10))
      cat(format(sprintf('%1.3f',Pv[k]),           width=10))
      cat(format(sprintf('%4.3f',CI[1,k]),         width=10))
      cat(format(sprintf('%4.3f',CI[2,k]),         width=10))
      cat(format(sprintf('%4.3f',H[1,k]),          width=9))
      cat(format(sprintf('%4.3f',H[2,k]),          width=10))
      cat(format(sprintf('%4.0f',Nh[1,k]+Nh[2,k]), width=10))
      cat('\n')
    }

    if (!is.null(xnorm)){
      cat(paste0(rep('-',80),collapse='')); cat('\n')
      cat(format('Pooled',                                   width=17))
      cat(format(sprintf('%7.3f',Coefs[cnum+1]),             width=10))
      cat(format(sprintf('%1.3f',Pv[cnum+1]),                width=10))
      cat(format(sprintf('%4.3f',CI[1,cnum+1]),              width=10))
      cat(format(sprintf('%4.3f',CI[2,cnum+1]),              width=10))
      cat(format(sprintf('%4.3f',H[1,cnum+1]),               width=9))
      cat(format(sprintf('%4.3f',H[2,cnum+1]),               width=10))
      cat(format(sprintf('%4.0f',Nh[1,cnum+1]+Nh[2,cnum+1]), width=10))

      cat('\n')
    }
  } else{
    for (k in 1:cnum){
      if (is.null(C2)){
        cat(format(sprintf('%4.2f',c.disp[k]),       width=17))
      } else {
        cat(paste0('(',format(sprintf('%4.2f',C[k])),',',format(sprintf('%4.2f',C2[k])),format(')',width=5)))
      }
      cat(format(sprintf('%7.3f',Coefs[k]),        width=10))
      cat(format(sprintf('%1.3f',Pv_cl[k]),        width=10))
      cat(format(sprintf('%4.3f',CI_cl[1,k]),      width=10))
      cat(format(sprintf('%4.3f',CI_cl[2,k]),      width=10))
      cat(format(sprintf('%4.3f',H[1,k]),          width=9))
      cat(format(sprintf('%4.3f',H[2,k]),          width=10))
      cat(format(sprintf('%4.0f',Nh[1,k]+Nh[2,k]), width=10))
      cat('\n')
    }

    if (!is.null(xnorm)){
      cat(paste0(rep('-',80),collapse='')); cat('\n')
      cat(format('Pooled',                                   width=17))
      cat(format(sprintf('%7.3f',Coefs[cnum+1]),             width=10))
      cat(format(sprintf('%1.3f',Pv_cl[cnum+1]),             width=10))
      cat(format(sprintf('%4.3f',CI_cl[1,cnum+1]),           width=10))
      cat(format(sprintf('%4.3f',CI_cl[2,cnum+1]),           width=10))
      cat(format(sprintf('%4.3f',H[1,cnum+1]),               width=9))
      cat(format(sprintf('%4.3f',H[2,cnum+1]),               width=10))
      cat(format(sprintf('%4.0f',Nh[1,cnum+1]+Nh[2,cnum+1]), width=10))

      cat('\n')
    }
  }

  cat(paste0(rep('=',80),collapse='')); cat('\n')


  #################################################################
  # Return values
  #################################################################

  colnames(B) <- c(1:cnum,"pooled")
  colnames(V) <- c(1:cnum,"pooled")
  colnames(Coefs) <- c(1:cnum,"pooled")
  colnames(CI) <- c(1:cnum,"pooled")
  colnames(Nh) <- c(1:cnum,"pooled")
  colnames(H) <- c(1:cnum,"pooled")
  colnames(Pv) <- c(1:cnum,"pooled")

  rownames(Nh) <- c("left","right")
  rownames(H) <- c("left","right")

  output <- list(B = B,
                V = V,
                Coefs = Coefs,
                V_cl = V_cl,
                Nh = Nh,
                CI = CI,
                CI_cl = CI_cl,
                H = H,
                Bbw = Bbw,
                Pv = Pv,
                Pv_cl = Pv_cl)

  return(output)

}

candidate_roots <- unique(c(
  Sys.getenv("RDMULTI_REPO_ROOT", unset = NA_character_),
  normalizePath(file.path(getwd(), "..", ".."), winslash = "/", mustWork = FALSE),
  normalizePath(getwd(), winslash = "/", mustWork = FALSE)
))
candidate_roots <- candidate_roots[!is.na(candidate_roots)]

repo_root <- NA_character_
for (root in candidate_roots) {
  if (file.exists(file.path(root, "R", "simdata_multic.csv"))) {
    repo_root <- root
    break
  }
}

if (is.na(repo_root)) {
  message("Skipping rdmulti numerical contracts: repository illustration data not found.")
  quit(save = "no", status = 0)
}

suppressPackageStartupMessages(library(rdmulti))

assert_allclose <- function(actual, expected, tolerance = 1e-8) {
  comparison <- all.equal(
    unname(actual),
    expected,
    tolerance = tolerance,
    check.attributes = FALSE
  )
  if (!isTRUE(comparison)) {
    stop(comparison, call. = FALSE)
  }
}

data <- read.csv(file.path(repo_root, "R", "simdata_multic.csv"))
invisible(capture.output(result <- rdmc(data$y, data$x, data$c)))

assert_allclose(
  result$Coefs,
  matrix(c(484.83090323, 297.98107945, 398.91490949, 436.40049184), nrow = 1)
)
assert_allclose(
  result$Pv,
  matrix(c(7.84407236e-48, 8.54527540e-16, 0, 7.41987105e-04), nrow = 1)
)
assert_allclose(
  result$H,
  matrix(
    c(
      14.6619828, 14.6619828,
      11.9522678, 11.9522678,
      NA_real_, NA_real_,
      13.6840172, 13.6840172
    ),
    nrow = 2
  )
)
assert_allclose(
  result$Nh,
  matrix(c(149, 140, 120, 126, 269, 266, 273, 277), nrow = 2)
)

data_cumul <- read.csv(file.path(repo_root, "R", "simdata_cumul.csv"))
cvec <- c(data_cumul$c[1], data_cumul$c[2])
invisible(capture.output(result_cumul <- rdms(data_cumul$y, data_cumul$x, cvec)))

assert_allclose(
  result_cumul$Coefs,
  matrix(c(395.4918261, 342.87249624, NA_real_), nrow = 1)
)
assert_allclose(
  result_cumul$Pv,
  matrix(c(1.59995659e-145, 3.42385007e-120, NA_real_), nrow = 1)
)
assert_allclose(
  result_cumul$H,
  matrix(c(15.10927597, 15.10927597, 12.22129226, 12.22129226, NA_real_, NA_real_), nrow = 2)
)
assert_allclose(
  result_cumul$Nh,
  matrix(c(140, 146, 142, 123, NA_real_, NA_real_), nrow = 2)
)

###################################################################
# rdmcplot: data-driven RD plots for multi-cutoff designs
# !version 2.0.0 15-May-2026
# Authors: Matias Cattaneo, Rocio Titiunik, Gonzalo Vazquez-Bare
###################################################################

#' Data-driven RD plots for multi-cutoff designs
#'
#' \code{rdmcplot()} implements data-driven Regression Discontinuity (RD) plots
#' for designs with multiple cutoffs.
#'
#'
#' @author
#' Matias D. Cattaneo, Princeton University. \email{matias.d.cattaneo@gmail.com}
#'
#' Rocio Titiunik, Princeton University. \email{rocio.titiunik@gmail.com}
#'
#' Gonzalo Vazquez-Bare, UC Santa Barbara. \email{gvazquezbare@gmail.com}
#'
#' @references
#'
#' Cattaneo, M.D., R. Titiunik and G. Vazquez-Bare. (2020). \href{https://rdpackages.github.io/references/Cattaneo-Titiunik-VazquezBare_2020_Stata.pdf}{Analysis of Regression Discontinuity Designs with Multiple Cutoffs or Multiple Scores}. \emph{Stata Journal}, forthcoming.
#'
#'
#' @param Y outcome variable.
#' @param X running variable.
#' @param C cutoff variable.
#' @param nbinsmat matrix of cutoff-specific number of bins. See \code{rdplot()}
#'   for details.
#' @param binselectvec vector of cutoff-specific bins selection method. See
#'   \code{rdplot()} for details.
#' @param scalevec vector of cutoff-specific scale factors. See \code{rdplot()}
#'   for details.
#' @param supportmat matrix of cutoff-specific support conditions. See
#'   \code{rdplot()} for details..
#' @param pvec vector of cutoff-specific polynomial orders. See \code{rdplot()}
#'   for details.
#' @param hmat matrix of cutoff-specific bandwidths. See \code{rdplot()} for
#'   details.
#' @param kernelvec vector of cutoff-specific kernels. See \code{rdplot()} for
#'   details.
#' @param weightsvec vector of cutoff-specific weights. See \code{rdplot()} for
#'   details.
#' @param covs_mat matrix of covariates. See \code{rdplot()} for
#'   details.
#' @param covs_list list of of covariates to be used in each cutoff.
#' @param covs_evalvec vector indicating the evaluation point for additional
#'   covariates. See \code{rdrobust()} for details.
#' @param covs_dropvec vector indicating whether collinear covariates should be
#'   dropped at each cutoff. See \code{rdrobust()} for details.
#' @param ginv.tolvec vector of cutoff-specific tolerances for generalized
#'   inverse calculations. See \code{rdplot()} for details.
#' @param masspointsvec vector indicating how to handle repeated values at each
#'   cutoff. See \code{rdplot()} for details.
#' @param ci adds confidence intervals of the specified level to the plot. See
#'   \code{rdrobust()} for details.
#' @param shade passes the shaded confidence interval option to \code{rdplot()}.
#' @param col_bins vector of colors for bins.
#' @param pch_bins vector of characters (pch) type for bins.
#' @param col_poly vector of colors for polynomial curves.
#' @param lty_poly vector of lty for polynomial curves.
#' @param col_xline vector of colors for vertical lines.
#' @param lty_xline vector of lty for vertical lines.
#' @param nobins omits bins plot.
#' @param nopoly omits polynomial curve plot.
#' @param noxline omits vertical lines indicating the cutoffs.
#' @param nodraw omits plot.
#' @param subset optional subset of observations to use.
#' @param data optional data frame for resolving string variable names.
#'
#'
#' @return
#' \item{clist}{list of cutoffs}
#' \item{cnum}{number of cutoffs}
#' \item{X0}{matrix of X values for control units}
#' \item{X1}{matrix of X values for treated units}
#' \item{Yhat0}{estimated polynomial for control units}
#' \item{Yhat1}{estimated polynomial for treated units}
#' \item{Xmean}{bin average of X values}
#' \item{Ymean}{bin average for Y values}
#' \item{CI_l}{lower end of confidence intervals}
#' \item{CI_r}{upper end of confidence intervals}
#' \item{cfail}{Cutoffs where rdrobust() encountered problems}

#'
#'
#' @examples
#' # Toy dataset
#' X <- runif(1000,0,100)
#' C <- c(rep(33,500),rep(66,500))
#' Y <- (1 + X + (X>=C))*(C==33)+(.5 + .5*X + .8*(X>=C))*(C==66) + rnorm(1000)
#' # rdmcplot with standard syntax
#' tmp <- rdmcplot(Y,X,C)
#'
#'
#' @export

rdmcplot <- function(Y,X,C,nbinsmat=NULL,binselectvec=NULL,scalevec=NULL,
                     supportmat=NULL,pvec=NULL,hmat=NULL,kernelvec=NULL,
                     weightsvec=NULL,covs_mat=NULL,covs_list=NULL,covs_evalvec=NULL,
                     covs_dropvec=NULL,ginv.tolvec=NULL,masspointsvec=NULL,
                     ci=NULL,shade=FALSE,col_bins=NULL,pch_bins=NULL,
                     col_poly=NULL,lty_poly=NULL,col_xline=NULL,lty_xline=NULL,
                     nobins=FALSE,nopoly=FALSE,noxline=FALSE,nodraw=FALSE,
                     subset=NULL,data=NULL){

  #################################################################
  # Setup and error checking
  #################################################################

  if (!is.null(data)){
    if (is.character(Y) && length(Y)==1) Y <- data[[Y]]
    if (is.character(X) && length(X)==1) X <- data[[X]]
    if (is.character(C) && length(C)==1) C <- data[[C]]
    if (!is.null(covs_mat) && is.character(covs_mat)) covs_mat <- data[,covs_mat,drop=FALSE]
  }

  n.orig <- length(Y)
  if (!is.null(subset)){
    Y <- Y[subset]
    X <- X[subset]
    C <- C[subset]
    if (!is.null(covs_mat) && nrow(as.matrix(covs_mat))==n.orig) covs_mat <- as.matrix(covs_mat)[subset,,drop=FALSE]
  }

  if (!is.numeric(C)){stop('C has to be numeric')}
  if (max(C,na.rm=TRUE)>=max(X,na.rm=TRUE) | min(C,na.rm=TRUE)<=min(X,na.rm=TRUE)){stop('cutoff variable outside range of running variable')}

  clist <- sort(unique(C))
  cnum <- length(clist)

  D <- as.numeric(X>=C)

  if (is.null(pvec)){pvec = rep(4,cnum)}
  if (!is.null(hmat)){
    if (is.null(dim(hmat))){
      hmat <- matrix(hmat,nrow=cnum,ncol=2)
    }
    haux <- hmat
  }  else{
    haux <- matrix(Inf,ncol=2,nrow=cnum)
  }
  if (!is.null(nbinsmat)){
    if (is.null(dim(nbinsmat))){
      nbinsmat <- matrix(nbinsmat,nrow=cnum,ncol=2)
    }
  }
  if (!is.null(supportmat)){
    if (is.null(dim(supportmat))){
      supportmat <- matrix(supportmat,nrow=cnum,ncol=2)
    }
  } else{
    supportmat <- matrix(NA,nrow=cnum,ncol=2)
  }
  if (is.null(binselectvec)) binselectvec <- rep('esmv',cnum)
  if (is.null(scalevec)) scalevec <- rep(1,cnum)
  if (is.null(kernelvec)) kernelvec <- rep('uni',cnum)
  if (is.null(covs_evalvec)) covs_evalvec <- rep('mean',cnum)
  if (is.null(covs_dropvec)) covs_dropvec <- rep(TRUE,cnum)
  if (is.null(ginv.tolvec)) ginv.tolvec <- rep(1e-20,cnum)
  if (is.null(masspointsvec)) masspointsvec <- rep('adjust',cnum)
  if (!is.null(covs_mat)){
    covs_mat <- as.matrix(covs_mat)
    if (!is.null(covs_list)){
      if (length(covs_list)!=cnum) stop('Elements in covs_list should equal number of cutoffs')
    }
  }

  X0 <- matrix(NA,nrow=length(Y),ncol=cnum)
  X1 <- matrix(NA,nrow=length(Y),ncol=cnum)
  YHAT0 <- matrix(NA,nrow=length(Y),ncol=cnum)
  YHAT1 <- matrix(NA,nrow=length(Y),ncol=cnum)
  XMEAN <- matrix(NA,nrow=length(Y),ncol=cnum)
  YMEAN <- matrix(NA,nrow=length(Y),ncol=cnum)
  CI_l <- matrix(NA,nrow=length(Y),ncol=cnum)
  CI_r <- matrix(NA,nrow=length(Y),ncol=cnum)
  Cfail <- numeric()


  #################################################################
  # Construct variables for plots
  #################################################################

  count <- 1
  count_fail <- 0
  for (c in clist){

    use <- C==c & X<=c+haux[count,2] & X>=c-haux[count,1]
    yc <- Y[use]
    xc <- X[use]
    dc <- D[use]
    yc0 <- yc[dc==0]
    yc1 <- yc[dc==1]
    xc0 <- xc[dc==0]
    xc1 <- xc[dc==1]

    if (!is.null(covs_mat)){
      covs_mat_c <- covs_mat[use,,drop=FALSE]
      if (!is.null(covs_list)){
        covs_aux <- covs_mat_c[,covs_list[[count]]]
      } else{
        covs_aux <- covs_mat_c
      }
    } else {
      covs_aux <- NULL
    }

    aux <- try(rdrobust::rdplot(yc,xc,c=c,
                                nbins=nbinsmat[count,],
                                binselect=binselectvec[count],
                                scale=scalevec[count],
                                p=pvec[count],
                                h=hmat[count,],
                                kernel=kernelvec[count],
                                weights=weightsvec[count],
                                covs=covs_aux,
                                covs_eval=covs_evalvec[count],
                                covs_drop=covs_dropvec[count],
                                ginv.tol=ginv.tolvec[count],
                                support=if (all(is.na(supportmat[count,]))) NULL else supportmat[count,],
                                masspoints=masspointsvec[count],
                                ci=ci,
                                shade=shade,
                                hide=TRUE),
               silent=TRUE)

    #if (class(aux)!="try-error"){
    if (!inherits(aux,'try-error')){
      xmean <- aux$vars_bins[,2]
      ymean <- aux$vars_bins[,3]

      xmean <- xmean[!is.na(xmean)]
      ymean <- ymean[!is.na(ymean)]

      x0 <- aux$vars_poly[aux$vars_poly[,1]<c,1]
      yhat0 <- aux$vars_poly[aux$vars_poly[,1]<c,2]
      x1 <- aux$vars_poly[aux$vars_poly[,1]>c,1]
      yhat1 <- aux$vars_poly[aux$vars_poly[,1]>c,2]

      length(xmean) <- length(Y)
      length(ymean) <- length(Y)
      length(x0) <- length(Y)
      length(yhat0) <- length(Y)
      length(x1) <- length(Y)
      length(yhat1) <- length(Y)

      XMEAN[,count] <- xmean
      YMEAN[,count] <- ymean
      X0[,count] <- x0
      X1[,count] <- x1
      YHAT0[,count] <- yhat0
      YHAT1[,count] <- yhat1

      if(!is.null(ci)){
        ci_l <- aux$vars_bins[,8]
        ci_r <- aux$vars_bins[,9]
        length(ci_l) <- length(Y)
        length(ci_r) <- length(Y)
        CI_l[,count] <- ci_l
        CI_r[,count] <- ci_r
      }
    } else{
      Cfail <- c(Cfail,c)
      count_fail <- count_fail + 1
    }

    count <- count + 1

  }

  Xmean <- data.frame(XMEAN)
  Xmean <- Xmean[1:max(colSums(!is.na(Xmean))),]
  names(Xmean) <- paste0(rep("Xmean"),1:cnum)
  Ymean <- data.frame(YMEAN)
  Ymean <- Ymean[1:max(colSums(!is.na(Ymean))),]
  names(Ymean) <- paste0(rep("Ymean"),1:cnum)
  X0    <- data.frame(X0)
  X0    <- X0[1:max(colSums(!is.na(X0))),]
  names(X0) <- paste0(rep("X0_"),1:cnum)
  X1    <- data.frame(X1)
  X1    <- X1[1:max(colSums(!is.na(X1))),]
  names(X1) <- paste0(rep("X1_"),1:cnum)
  Yhat0    <- data.frame(YHAT0)
  Yhat0    <- Yhat0[1:max(colSums(!is.na(Yhat0))),]
  names(Yhat0) <- paste0(rep("Yhat0_"),1:cnum)
  Yhat1    <- data.frame(YHAT1)
  Yhat1    <- Yhat1[1:max(colSums(!is.na(Yhat1))),]
  names(Yhat1) <- paste0(rep("Yhat1_"),1:cnum)
  if(!is.null(ci)){
    CI_l <- data.frame(CI_l)
    CI_l <- CI_l[1:max(colSums(!is.na(CI_l))),]
    names(CI_l) <- paste0(rep("CI_l_"),1:cnum)
    CI_r <- data.frame(CI_r)
    CI_r <- CI_r[1:max(colSums(!is.na(CI_r))),]
    names(CI_r) <- paste0(rep("CI_r_"),1:cnum)
  }


  #################################################################
  # Plots
  #################################################################

  colorlist <- c('darkblue','darkred','darkgreen','darkorange','gray50','khaki4','brown3','blue','darkgoldenrod4','cyan4')

  if (is.null(col_bins)){
    col_bins <- colorlist
  }
  if (is.null(pch_bins)){
    pch_bins <- rep(1,cnum)
  }
  if (is.null(col_poly)){
    col_poly <- colorlist
  }
  if (is.null(lty_poly)){
    lty_poly <- rep('solid',cnum)
  }
  if (is.null(col_xline)){
    col_xline <- colorlist
  }
  if (is.null(lty_xline)){
    lty_xline <- rep('dashed',cnum)
  }

  rdmc_plot <- ggplot() + theme_bw() + labs(x='Running variable', y='Outcome')

  if (nobins==FALSE){
    for (c in 1:cnum){
      plot_data <- data.frame(x=Xmean[,c], y=Ymean[,c])
      rdmc_plot <- rdmc_plot + geom_point(data=plot_data, aes(x=.data$x,y=.data$y), col=col_bins[c], shape=pch_bins[c], na.rm=TRUE)
    }
  }
  if (!is.null(ci)){
    for (c in 1:cnum){
      ci_data <- data.frame(x=Xmean[,c], ymin=CI_l[,c], ymax=CI_r[,c])
      rdmc_plot <- rdmc_plot + geom_errorbar(data=ci_data, aes(x=.data$x,ymin=.data$ymin,ymax=.data$ymax), col=col_bins[c], linetype=1)
    }
  }
  if (nopoly==FALSE){
    for (c in 1:cnum){
      poly_l <- data.frame(x=X0[,c], y=Yhat0[,c])
      poly_r <- data.frame(x=X1[,c], y=Yhat1[,c])
      rdmc_plot <- rdmc_plot + geom_line(data=poly_l, aes(x=.data$x,y=.data$y), col=col_poly[c], linetype=lty_poly[c], na.rm=TRUE) +
        geom_line(data=poly_r, aes(x=.data$x,y=.data$y), col=col_poly[c], linetype=lty_poly[c], na.rm=TRUE)
    }
  }
  if (noxline==FALSE){
    for (c in 1:cnum){
      rdmc_plot <- rdmc_plot + geom_vline(xintercept=clist[c],col=col_xline[c],linetype=lty_xline[c])
    }
  }

  if (nodraw==FALSE){
    print(rdmc_plot)
  }

  if (count_fail>0){
    warning("rdplot() could not run in one or more cutoffs.")
  }

  #################################################################
  # Return values
  #################################################################

  if (is.null(ci)){
    output <- list(clist = clist,
                  cnum = cnum,
                  X0 = X0,
                  X1 = X1,
                  Yhat0 = Yhat0,
                  Yhat1 = Yhat1,
                  Xmean = Xmean,
                  Ymean = Ymean,
                  rdmc_plot = rdmc_plot,
                  cfail = Cfail)
  }  else {
    output <- list(clist = clist,
                  cnum = cnum,
                  X0 = X0,
                  X1 = X1,
                  Yhat0 = Yhat0,
                  Yhat1 = Yhat1,
                  Xmean = Xmean,
                  Ymean = Ymean,
                  rdmc_plot = rdmc_plot,
                  CI_l = CI_l,
                  CI_r = CI_r,
                  cfail = Cfail)
  }


  return(output)
}

###################################################################
# rdmc: point estimation and robust bias-corrected inference for multi-cutoff designs
# !version 2.0.0 15-May-2026
# Authors: Matias Cattaneo, Rocio Titiunik, Gonzalo Vazquez-Bare
###################################################################

#' Point estimation and robust bias-corrected inference for multi-cutoff designs
#'
#' \code{rdmc()} implements point estimation and robust bias-corrected inference
#' for Regression Discontinuity (RD) designs with multiple cutoffs.
#'
#'
#' @author
#' Matias D. Cattaneo, Princeton University. \email{matias.d.cattaneo@gmail.com}
#'
#' Rocio Titiunik, Princeton University. \email{rocio.titiunik@gmail.com}
#'
#' Gonzalo Vazquez-Bare, UC Santa Barbara. \email{gvazquezbare@gmail.com}
#'
#' @references
#'
#' Cattaneo, M.D., R. Titiunik and G. Vazquez-Bare. (2020). \href{https://rdpackages.github.io/references/Cattaneo-Titiunik-VazquezBare_2020_Stata.pdf}{Analysis of Regression Discontinuity Designs with Multiple Cutoffs or Multiple Scores}. \emph{Stata Journal}, forthcoming.
#'
#'
#' @param Y outcome variable.
#' @param X running variable.
#' @param C cutoff variable.
#' @param fuzzy specifies a fuzzy design. See \code{rdrobust()} for details.
#' @param derivvec vector of cutoff-specific order of derivatives. See
#'   \code{rdrobust()} for details.
#' @param pooled_opt options to be passed to \code{rdrobust()} to calculate
#'   pooled estimand.
#' @param verbose displays the output from \code{rdrobust} for estimating the
#'   pooled estimand.
#' @param pvec vector of cutoff-specific polynomial orders. See
#'   \code{rdrobust()} for details.
#' @param qvec vector of cutoff-specific polynomial orders for bias estimation.
#'   See \code{rdrobust()} for details.
#' @param hmat matrix of cutoff-specific bandwidths. See \code{rdrobust()} for
#'   details.
#' @param bmat matrix of cutoff-specific bandwidths for bias estimation. See
#'   \code{rdrobust()} for details.
#' @param rhovec vector of cutoff-specific values of rho. See \code{rdrobust()}
#'   for details.
#' @param covs_mat matrix of covariates. See \code{rdrobust()}
#'   for details.
#' @param covs_list list of covariates to be used in each cutoff.
#' @param covs_dropvec vector indicating whether collinear covariates should be
#'   dropped at each cutoff. See \code{rdrobust()} for details.
#' @param kernelvec vector of cutoff-specific kernels. See \code{rdrobust()} for
#'   details.
#' @param weightsvec vector of length equal to the number of cutoffs indicating
#'   the names of the variables to be used as weights in each cutoff. See \code{rdrobust()} for details.
#' @param bwselectvec vector of cutoff-specific bandwidth selection methods. See
#'   \code{rdrobust()} for details.
#' @param scaleparvec vector of cutoff-specific scale parameters. See
#'   \code{rdrobust()} for details.
#' @param scaleregulvec vector of cutoff-specific scale regularization
#'   parameters. See \code{rdrobust()} for details.
#' @param masspointsvec vector indicating how to handle repeated values at each
#'   cutoff. See \code{rdrobust()} for details.
#' @param bwcheckvec vector indicating the value of bwcheck at each cutoff. See
#'   \code{rdrobust()} for details.
#' @param bwrestrictvec vector indicating whether computed bandwidths are
#'   restricted to the range or runvar at each cutoff. See \code{rdrobust()} for
#'   details.
#' @param stdvarsvec vector indicating whether variables are standardized at
#'   each cutoff. See \code{rdrobust()} for details.
#' @param vcevec vector of cutoff-specific variance-covariance estimation
#'   methods. See \code{rdrobust()} for details.
#' @param nnmatchvec vector of cutoff-specific nearest neighbors for variance
#'   estimation. See \code{rdrobust()} for details.
#' @param cluster cluster ID variable. See \code{rdrobust()} for details.
#' @param ginv.tolvec vector of cutoff-specific tolerances for generalized
#'   inverse calculations. See \code{rdrobust()} for details.
#' @param sharpbwvec vector indicating whether fuzzy RD bandwidth selection
#'   uses the sharp RD model at each cutoff. See \code{rdrobust()} for details.
#' @param level confidence level for confidence intervals. See \code{rdrobust()}
#'   for details.
#' @param plot plots cutoff-specific estimates and weights.
#' @param conventional reports conventional, instead of robust-bias corrected,
#'   p-values and confidence intervals.
#' @param subset optional subset of observations to use.
#' @param data optional data frame for resolving string variable names.
#'
#'
#' @return
#' \item{tau}{pooled estimate}
#' \item{se.rb}{robust bias corrected standard error for pooled estimate}
#' \item{pv.rb}{robust bias corrected p-value for pooled estimate}
#' \item{ci.rb.l}{left limit of robust bias corrected CI for pooled estimate}
#' \item{ci.rb.r}{right limit of robust bias corrected CI for pooled estimate}
#' \item{hl}{bandwidth to the left of the cutoff for pooled estimate}
#' \item{hr}{bandwidth to the right of the cutofffor pooled estimate}
#' \item{Nhl}{sample size within bandwidth to the left of the cutoff for pooled estimate}
#' \item{Nhr}{sample size within bandwidth to the right of the cutoff for pooled estimate}
#' \item{B}{vector of bias-corrected estimates}
#' \item{V}{vector of robust variances of the estimates}
#' \item{Coefs}{vector of conventional estimates}
#' \item{W}{vector of weights for each cutoff-specific estimate}
#' \item{Nh}{vector of sample sizes within bandwidth}
#' \item{CI}{robust bias-corrected confidence intervals}
#' \item{H}{matrix of bandwidths}
#' \item{Pv}{vector of robust p-values}
#' \item{rdrobust.results}{results from rdrobust for pooled estimate}
#' \item{cfail}{Cutoffs where rdrobust() encountered problems}
#'
#'
#' @examples
#' # Toy dataset
#' X <- runif(1000,0,100)
#' C <- c(rep(33,500),rep(66,500))
#' Y <- (1 + X + (X>=C))*(C==33)+(.5 + .5*X + .8*(X>=C))*(C==66) + rnorm(1000)
#' # rdmc with standard syntax
#' tmp <- rdmc(Y,X,C)
#'
#'
#' @export

rdmc <- function(Y,X,C,fuzzy=NULL,derivvec=NULL,pooled_opt=NULL,verbose=FALSE,
                pvec=NULL,qvec=NULL,hmat=NULL,bmat=NULL,rhovec=NULL,
                covs_mat=NULL,covs_list=NULL,covs_dropvec=NULL,kernelvec=NULL,weightsvec=NULL,
                bwselectvec=NULL,scaleparvec=NULL,scaleregulvec=NULL,
                masspointsvec=NULL,bwcheckvec=NULL,bwrestrictvec=NULL,
                stdvarsvec=NULL,vcevec=NULL,nnmatchvec=NULL,cluster=NULL,
                ginv.tolvec=NULL,sharpbwvec=NULL,level=95,plot=FALSE,
                conventional=FALSE,subset=NULL,data=NULL){

  #################################################################
  # Setup and error checking
  #################################################################

  if (!is.null(data)){
    if (is.character(Y) && length(Y)==1) Y <- data[[Y]]
    if (is.character(X) && length(X)==1) X <- data[[X]]
    if (is.character(C) && length(C)==1) C <- data[[C]]
    if (!is.null(fuzzy) && is.character(fuzzy) && length(fuzzy)==1) fuzzy <- data[[fuzzy]]
    if (!is.null(cluster) && is.character(cluster) && length(cluster)==1) cluster <- data[[cluster]]
    if (!is.null(covs_mat) && is.character(covs_mat)) covs_mat <- data[,covs_mat,drop=FALSE]
  }

  n.orig <- length(Y)
  if (!is.null(subset)){
    Y <- Y[subset]
    X <- X[subset]
    C <- C[subset]
    if (!is.null(fuzzy) && length(fuzzy)==n.orig) fuzzy <- fuzzy[subset]
    if (!is.null(cluster) && length(cluster)==n.orig) cluster <- cluster[subset]
    if (!is.null(covs_mat) && nrow(as.matrix(covs_mat))==n.orig) covs_mat <- as.matrix(covs_mat)[subset,,drop=FALSE]
  }

  if (!is.numeric(C)){stop('C has to be numeric')}
  if (max(C,na.rm=TRUE)>=max(X,na.rm=TRUE) | min(C,na.rm=TRUE)<=min(X,na.rm=TRUE)){stop('cutoff variable outside range of running variable')}

  clist <- sort(unique(C))
  cnum <- length(clist)

  Xc <- X - C

  if (!is.null(hmat)){
    if (is.null(dim(hmat))){
      hmat <- matrix(hmat,nrow=cnum,ncol=2)
    }
  }
  if (!is.null(bmat)){
    if (is.null(dim(bmat))){
      bmat <- matrix(bmat,nrow=cnum,ncol=2)
    }
  }
  if (is.null(covs_dropvec)) covs_dropvec <- rep(TRUE,cnum)
  if (is.null(kernelvec)) kernelvec <- rep('tri',cnum)
  if (is.null(bwselectvec)) bwselectvec <- rep('mserd',cnum)
  if (is.null(vcevec)) vcevec <- rep('nn',cnum)
  if (is.null(nnmatchvec)) nnmatchvec <- rep(3,cnum)
  if (is.null(scaleparvec)) scaleparvec <- rep(1,cnum)
  if (is.null(scaleregulvec)) scaleregulvec <- rep(1,cnum)
  if (is.null(masspointsvec)) masspointsvec <- rep('adjust',cnum)
  if (is.null(bwrestrictvec)) bwrestrictvec <- rep(TRUE,cnum)
  if (is.null(stdvarsvec)) stdvarsvec <- rep(FALSE,cnum)
  if (is.null(ginv.tolvec)) ginv.tolvec <- rep(1e-20,cnum)
  if (is.null(sharpbwvec)) sharpbwvec <- rep(FALSE,cnum)
  if (!is.null(covs_mat)){
    covs_mat <- as.matrix(covs_mat)
    if (!is.null(covs_list)){
      if (length(covs_list)!=cnum) stop('Elements in covs_list should equal number of cutoffs')
    }
  }

  B <- matrix(NA,nrow=1,ncol=cnum+2)
  V <- matrix(NA,nrow=1,ncol=cnum+2)
  V_cl <- matrix(NA,nrow=1,ncol=cnum+2)
  Coefs <- matrix(NA,nrow=1,ncol=cnum+2)
  Nh <- matrix(NA,nrow=2,ncol=cnum+2)
  CI <- matrix(NA,nrow=2,ncol=cnum+2)
  CI_cl <- matrix(NA,nrow=2,ncol=cnum+2)
  Pv <- matrix(NA,nrow=1,ncol=cnum+2)
  Pv_cl <- matrix(NA,nrow=1,ncol=cnum+2)
  H <- matrix(NA,nrow=2,ncol=cnum+2)
  Bbw <- matrix(NA,nrow=2,ncol=cnum+2)
  W <- matrix(NA,nrow=1,ncol=cnum)
  Cfail <- numeric()

  #################################################################
  # Calculate pooled estimate
  #################################################################

  aux1 <- paste0('rdrobust::rdrobust(Y,Xc,,fuzzy=fuzzy,',pooled_opt,')')

  rdr <- eval(parse(text=aux1))

  hl <- rdr$bws[1,1]
  hr <- rdr$bws[1,2]
  Nhl <- rdr$N_h[1]
  Nhr <- rdr$N_h[2]

  B[1,cnum+2] <- rdr$Estimate[2]
  V[1,cnum+2] <- rdr$se[3]^2
  Coefs[1,cnum+2] <- rdr$Estimate[1]
  V_cl[1,cnum+2] <- rdr$se[1]^2
  CI[,cnum+2] <- rdr$ci[3,]
  CI_cl[,cnum+2] <- rdr$ci[1,]
  H[,cnum+2] <- rdr$bws[1,]
  Bbw[,cnum+2] <- rdr$bws[2,]
  Nh[,cnum+2] <- rdr$N_h
  Pv[1,cnum+2] <- rdr$pv[3]
  Pv_cl[1,cnum+2] <- rdr$pv[1]

  #################################################################
  # Calculate cutoff-specific estimates and weights
  #################################################################

  count <- 1
  count_fail <- 0
  for (c in clist){

    yc <- Y[abs(C-c)<=.Machine$double.eps]
    xc <- Xc[abs(C-c)<=.Machine$double.eps]
    if (!is.null(fuzzy)){
      fuzzyc <- fuzzy[abs(C-c)<=.Machine$double.eps]
    } else {
      fuzzyc <- NULL
    }
    if (!is.null(cluster)){
      cc <- cluster[abs(C-c)<=.Machine$double.eps]
    } else {
      cc <- NULL
    }

    if (!is.null(weightsvec)){
      weightaux <- weightsvec[count]
      weights_source <- eval(parse(text=weightaux), parent.frame())
      weightsc <- weights_source[abs(C-c)<=.Machine$double.eps]
    } else{
      weightsc <- NULL
    }

    if (!is.null(covs_mat)){
      covs_mat_c <- covs_mat[abs(C-c)<=.Machine$double.eps,]
      if (!is.null(covs_list)){
        covs_aux <- covs_mat_c[,covs_list[[count]]]
      } else{
        covs_aux <- covs_mat_c
      }
    } else {
      covs_aux <- NULL
    }

    rdr.tmp <- try(rdrobust::rdrobust(yc,xc,
                                      fuzzy=fuzzyc,
                                      deriv=derivvec[count],
                                      p=pvec[count],
                                      q=qvec[count],
                                      h=hmat[count,],
                                      b=bmat[count,],
                                      rho=rhovec[count],
                                      covs=covs_aux,
                                      covs_drop=covs_dropvec[count],
                                      ginv.tol=ginv.tolvec[count],
                                      kernel=kernelvec[count],
                                      weights=weightsc,
                                      bwselect=bwselectvec[count],
                                      scalepar=scaleparvec[count],
                                      scaleregul=scaleregulvec[count],
                                      masspoints=masspointsvec[count],
                                      bwcheck=bwcheckvec[count],
                                      bwrestrict=bwrestrictvec[count],
                                      stdvars=stdvarsvec[count],
                                      vce=vcevec[count],
                                      nnmatch=nnmatchvec[count],
                                      cluster=cc,
                                      sharpbw=sharpbwvec[count],
                                      level=level),
                   silent=TRUE)

    #if (class(rdr.tmp)!="try-error"){
    if (!inherits(rdr.tmp,'try-error')){
      B[1,count] <- rdr.tmp$Estimate[2]
      V[1,count] <- rdr.tmp$se[3]^2
      Coefs[1,count] <- rdr.tmp$Estimate[1]
      V_cl[1,count] <- rdr.tmp$se[1]^2
      CI[,count] <- rdr.tmp$ci[3,]
      CI_cl[,count] <- rdr.tmp$ci[1,]
      H[,count] <- rdr.tmp$bws[1,]
      Bbw[,count] <- rdr.tmp$bws[2,]
      Nh[,count] <- rdr.tmp$N_h
      Pv[1,count] <- rdr.tmp$pv[3]
      Pv_cl[1,count] <- rdr.tmp$pv[1]
    } else{
      Cfail <- c(Cfail,c)
      count_fail <- count_fail + 1
    }

    count <- count + 1

  }

  # Weights

  W[1,] <- colSums(Nh[,1:cnum],na.rm=TRUE)/sum(Nh[,1:cnum],na.rm=TRUE)
  W[is.na(W)] <- 0

  # Weighted estimate

  Baux <- B
  Vaux <- V
  Coefsaux <- Coefs
  Vaux_cl <- V_cl
  Baux[is.na(Baux)] <- 0
  Vaux[is.na(Vaux)] <- 0
  Coefsaux[is.na(Coefsaux)] <- 0
  Vaux_cl[is.na(Vaux_cl)] <- 0

  B[1,cnum+1] <- Baux[1,1:cnum]%*%t(W)
  V[1,cnum+1] <- Vaux[1,1:cnum]%*%t(W^2)
  Coefs[1,cnum+1] <- Coefsaux[1,1:cnum]%*%t(W)
  V_cl[1,cnum+1] <- Vaux_cl[1,1:cnum]%*%t(W^2)

  Nh[,cnum+1] <- rowSums(Nh[,1:cnum],na.rm=TRUE)

  CI[1,cnum+1] <- B[1,cnum+1]-sqrt(V[1,cnum+1])*qnorm(1-(1-level/100)/2)
  CI[2,cnum+1] <- B[1,cnum+1]+sqrt(V[1,cnum+1])*qnorm(1-(1-level/100)/2)
  Pv[1,cnum+1] <- 2*(1-pnorm(abs(B[1,cnum+1]/sqrt(V[1,cnum+1]))))

  CI_cl[1,cnum+1] <- Coefs[1,cnum+1]-sqrt(V_cl[1,cnum+1])*qnorm(1-(1-level/100)/2)
  CI_cl[2,cnum+1] <- Coefs[1,cnum+1]+sqrt(V_cl[1,cnum+1])*qnorm(1-(1-level/100)/2)
  Pv_cl[1,cnum+1] <- 2*(1-pnorm(abs(Coefs[1,cnum+1]/sqrt(V_cl[1,cnum+1]))))

  #################################################################
  # Display results
  #################################################################

  if (verbose==TRUE){
    print(summary(rdr))
    cat('\n')
  }

  if (conventional==FALSE){
    cat('\n')
    cat('Cutoff-specific RD estimation with robust bias-corrected inference'); cat('\n')
    cat(paste0(rep('=',80),collapse='')); cat('\n')
    cat(format('Cutoff',             width=11))
    cat(format('Coef.',              width=8))
    cat(format('P-value',            width=16))
    cat(format(paste0(level,'% CI'), width=16))
    cat(format('hl',                 width=9))
    cat(format('hr',                 width=9))
    cat(format('Nh',                 width=5))
    cat(format('Weight',             width=5))
    cat('\n')
    cat(paste0(rep('=',80),collapse=''))
    cat('\n')

    for (k in 1:cnum){
      cat(format(sprintf('%4.3f',clist[k]),        width=11))
      cat(format(sprintf('%7.3f',Coefs[k]),        width=9))
      cat(format(sprintf('%1.3f',Pv[k]),           width=9))
      cat(format(sprintf('%4.3f',CI[1,k]),         width=10))
      cat(format(sprintf('%4.3f',CI[2,k]),         width=10))
      cat(format(sprintf('%4.3f',H[1,k]),          width=9))
      cat(format(sprintf('%4.3f',H[2,k]),          width=9))
      cat(format(sprintf('%4.0f',Nh[1,k]+Nh[2,k]), width=8))
      cat(format(sprintf('%1.3f',W[k]),            width=5))
      cat('\n')
    }
    cat(paste0(rep('-',80),collapse='')); cat('\n')

    cat(format('Weighted',                                 width=11))
    cat(format(sprintf('%7.3f',Coefs[1,cnum+1]),           width=9))
    cat(format(sprintf('%1.3f',Pv[1,cnum+1]),              width=9))
    cat(format(sprintf('%4.3f',CI[1,cnum+1]),              width=10))
    cat(format(sprintf('%4.3f',CI[2,cnum+1]),              width=13))
    cat(format('  .',                                      width=9))
    cat(format('  .',                                      width=6))
    cat(format(sprintf('%4.0f',Nh[1,cnum+1]+Nh[2,cnum+1]), width=11))
    cat(format(' .',                                       width=5))
    cat('\n')

    cat(format('Pooled',                                   width=11))
    cat(format(sprintf('%7.3f',Coefs[1,cnum+2]),           width=9))
    cat(format(sprintf('%1.3f',Pv[cnum+2]),                width=9))
    cat(format(sprintf('%4.3f',CI[1,cnum+2]),              width=10))
    cat(format(sprintf('%4.3f',CI[2,cnum+2]),              width=10))
    cat(format(sprintf('%4.3f',H[1,cnum+2]),               width=9))
    cat(format(sprintf('%4.3f',H[2,cnum+2]),               width=9))
    cat(format(sprintf('%4.0f',Nh[1,cnum+2]+Nh[2,cnum+2]), width=11))
    cat(format(' .',                                       width=5))
    cat('\n')
    cat(paste0(rep('=',80),collapse='')); cat('\n')
  } else{
    cat('\n')
    cat('Cutoff-specific RD estimation with conventional inference'); cat('\n')
    cat(paste0(rep('=',80),collapse='')); cat('\n')
    cat(format('Cutoff',  width=11))
    cat(format('Coef.',   width=8))
    cat(format('P-value', width=16))
    cat(format(paste0(level, '% CI'), width=16))
    cat(format('hl',      width=9))
    cat(format('hr',      width=9))
    cat(format('Nh',      width=5))
    cat(format('Weight',  width=5))
    cat('\n')
    cat(paste0(rep('=',80),collapse=''))
    cat('\n')

    for (k in 1:cnum){
      cat(format(sprintf('%4.3f',clist[k]),        width=11))
      cat(format(sprintf('%7.3f',Coefs[k]),        width=9))
      cat(format(sprintf('%1.3f',Pv_cl[k]),        width=9))
      cat(format(sprintf('%4.3f',CI_cl[1,k]),      width=10))
      cat(format(sprintf('%4.3f',CI_cl[2,k]),      width=10))
      cat(format(sprintf('%4.3f',H[1,k]),          width=9))
      cat(format(sprintf('%4.3f',H[2,k]),          width=9))
      cat(format(sprintf('%4.0f',Nh[1,k]+Nh[2,k]), width=8))
      cat(format(sprintf('%1.3f',W[k]),            width=5))
      cat('\n')
    }
    cat(paste0(rep('-',80),collapse='')); cat('\n')

    cat(format('Weighted',                                 width=11))
    cat(format(sprintf('%7.3f',Coefs[1,cnum+1]),           width=9))
    cat(format(sprintf('%1.3f',Pv_cl[1,cnum+1]),           width=9))
    cat(format(sprintf('%4.3f',CI_cl[1,cnum+1]),           width=10))
    cat(format(sprintf('%4.3f',CI_cl[2,cnum+1]),           width=13))
    cat(format('  .',                                      width=9))
    cat(format('  .',                                      width=6))
    cat(format(sprintf('%4.0f',Nh[1,cnum+1]+Nh[2,cnum+1]), width=11))
    cat(format(' .',                                       width=5))
    cat('\n')

    cat(format('Pooled',                                   width=11))
    cat(format(sprintf('%7.3f',Coefs[cnum+2]),             width=9))
    cat(format(sprintf('%1.3f',Pv_cl[cnum+2]),             width=9))
    cat(format(sprintf('%4.3f',CI_cl[1,cnum+2]),           width=10))
    cat(format(sprintf('%4.3f',CI_cl[2,cnum+2]),           width=10))
    cat(format(sprintf('%4.3f',H[1,cnum+2]),               width=9))
    cat(format(sprintf('%4.3f',H[2,cnum+2]),               width=9))
    cat(format(sprintf('%4.0f',Nh[1,cnum+2]+Nh[2,cnum+2]), width=11))
    cat(format(' .',                                       width=5))
    cat('\n')
    cat(paste0(rep('=',80),collapse='')); cat('\n')
  }

  if (count_fail>0){
    warning("rdrobust() could not run in one or more cutoffs.")
  }

  #################################################################
  # Plots
  #################################################################

  if (plot==TRUE){

    ylim <- c(min(CI,na.rm=TRUE)*1.3,max(CI,na.rm=TRUE)*1.3)
    xlim <- c(min(clist),max(clist))

    par(mfrow=c(1,2))

    plot(NA,ylim=ylim,xlim=xlim,ylab='Treatment effect',xlab='Cutoff')

    polygon(x=c(min(clist),min(clist),max(clist),max(clist)),y=c(CI[1,cnum+2],CI[2,cnum+2],CI[2,cnum+2],CI[1,cnum+2]),
            border = NA,col='gray87')
    polygon(x=c(min(clist),min(clist),max(clist),max(clist)),y=c(CI[1,cnum+1],CI[2,cnum+1],CI[2,cnum+1],CI[1,cnum+1]),
            border = NA,col=rgb(139/255,0,0,0.2))
    points(clist,Coefs[1:cnum],col='darkblue',pch=16)

    if (conventional==FALSE){
      arrows(clist,CI[1,1:cnum],clist,CI[2,1:cnum],length=0.05,angle=90,code=3)
    } else{
      arrows(clist,CI_cl[1,1:cnum],clist,CI_cl[2,1:cnum],length=0.05,angle=90,code=3)
    }

    abline(h=Coefs[1,cnum+2],col='gray34')
    abline(h=Coefs[1,cnum+1],col='darkred')
    abline(h=0,lty='dotted')

    legend('bottomright',legend=c('Estimates','Weighted estimate','Pooled estimate'),pch=c(16,NA,NA),lty=c(NA,1,1),
           col=c('darkblue','darkred','gray34'),bty='n',cex=0.75)

    mtext(paste0('Bars are ', level, '% CIs for estimates. \nShaded area is the ', level, '% CI for weighted and pooled.'), cex = 0.8)

    barplot(W,xlab='Cutoff',ylab='Weight',names.arg=clist,space=1.5)

  }

  #################################################################
  # Return values
  #################################################################

  colnames(B) <- c(1:cnum,"weighted","pooled")
  colnames(V) <- c(1:cnum,"weighted","pooled")
  colnames(Coefs) <- c(1:cnum,"weighted","pooled")
  colnames(CI) <- c(1:cnum,"weighted","pooled")
  colnames(CI_cl) <- c(1:cnum,"weighted","pooled")
  colnames(Nh) <- c(1:cnum,"weighted","pooled")
  colnames(H) <- c(1:cnum,"weighted","pooled")
  colnames(Bbw) <- c(1:cnum,"weighted","pooled")
  colnames(Pv) <- c(1:cnum,"weighted","pooled")
  colnames(Pv_cl) <- c(1:cnum,"weighted","pooled")

  rownames(Nh) <- c("left","right")
  rownames(H) <- c("left","right")

  output <- list(tau = rdr$Estimate[1],
                se.rb = rdr$se[3],
                pv.rb = rdr$pv[3],
                ci.rb.l = rdr$ci[3,1],
                ci.rb.r = rdr$ci[3,2],
                hl = rdr$bws[1,1],
                hr = rdr$bws[1,2],
                Nhl = rdr$N_h[1],
                Nhr = rdr$N_h[2],
                B = B,
                V = V,
                Coefs = Coefs,
                V_cl = V_cl,
                W = W,
                Nh = Nh,
                CI = CI,
                CI_cl = CI_cl,
                H = H,
                Bbw = Bbw,
                Pv = Pv,
                Pv_cl = Pv_cl,
                rdrobust.results = rdr,
                cfail = Cfail)

  return(output)

}

###################################################################
# rdmulti: Robust Local Polynomial Methods for RD designs with Multiple Cutoffs or Multiple Scores
# !version 2.0.0 15-May-2026
# Authors: Matias Cattaneo, Rocio Titiunik, Gonzalo Vazquez-Bare
###################################################################

#' Robust Local Polynomial Methods for RD designs with Multiple Cutoffs or Multiple Scores
#'
#' The package \code{rdmulti} implements estimation, inference, and graphical
#' procedures for Regression Discontinuity (RD) designs with multiple cutoffs or
#' multiple scores. \code{\link{rdmc}()} provides point estimation and robust
#' bias-corrected inference for multi-cutoff designs, \code{\link{rdmcplot}()}
#' provides data-driven RD plots for multi-cutoff designs, and
#' \code{\link{rdms}()} provides point estimation and robust bias-corrected
#' inference for multi-score designs. For more details, and related \code{Stata}
#' and \code{R} packages useful for analysis of RD designs, visit
#' \url{https://rdpackages.github.io/}.
#'
#' @author
#' Matias D. Cattaneo, Princeton University. \email{matias.d.cattaneo@gmail.com}
#'
#' Rocio Titiunik, Princeton University. \email{rocio.titiunik@gmail.com}
#'
#' Gonzalo Vazquez-Bare, UC Santa Barbara. \email{gvazquezbare@gmail.com}
#'
#' @references
#'
#' Calonico, S., M.D. Cattaneo, M. Farrell and R. Titiunik. (2017). \href{https://rdpackages.github.io/references/Calonico-Cattaneo-Farrell-Titiunik_2017_Stata.pdf}{ \code{rdrobust}: Software for Regression Discontinuity Designs}. \emph{Stata Journal} 17(2): 372-404.
#'
#' Calonico, S., M.D. Cattaneo, and R. Titiunik. (2014). \href{https://rdpackages.github.io/references/Calonico-Cattaneo-Titiunik_2014_Stata.pdf}{Robust Data-Driven Inference in the Regression-Discontinuity Design}. \emph{Stata Journal} 14(4): 909-946.
#'
#' Calonico, S., M.D. Cattaneo, and R. Titiunik. (2015). \href{https://rdpackages.github.io/references/Calonico-Cattaneo-Titiunik_2015_R.pdf}{ \code{rdrobust}: An R Package for Robust Nonparametric Inference in Regression-Discontinuity Designs}. \emph{R Journal} 7(1): 38-51.
#'
#' Cattaneo, M.D., L. Keele, R. Titiunik and G. Vazquez-Bare. (2016). \href{https://rdpackages.github.io/references/Cattaneo-Keele-Titiunik-VazquezBare_2016_JOP.pdf}{Interpreting Regression Discontinuity Designs with Multiple Cutoffs}. \emph{Journal of Politics} 78(4): 1229-1248.
#'
#' Cattaneo, M.D., L. Keele, R. Titiunik and G. Vazquez-Bare. (2020). \href{https://rdpackages.github.io/references/Cattaneo-Keele-Titiunik-VazquezBare_2021_JASA.pdf}{Extrapolating Treatment Effects in Multi-Cutoff Regression Discontinuity Designs}. \emph{Journal of the American Statistical Association} 116(536): 1941, 1952.
#'
#' Cattaneo, M.D., R. Titiunik and G. Vazquez-Bare. (2020). \href{https://rdpackages.github.io/references/Cattaneo-Titiunik-VazquezBare_2020_Stata.pdf}{Analysis of Regression Discontinuity Designs with Multiple Cutoffs or Multiple Scores}. \emph{Stata Journal} 20(4): 866-891.
#'
#' Keele, L. and R. Titiunik. (2015). \href{https://rdpackages.github.io/references/Keele-Titiunik_2015_PA.pdf}{Geographic Boundaries as Regression Discontinuities}. \emph{Political Analysis} 23(1): 127-155
#'
#' @importFrom graphics abline
#' @importFrom graphics arrows
#' @importFrom graphics legend
#' @importFrom graphics barplot
#' @importFrom graphics lines
#' @importFrom graphics mtext
#' @importFrom graphics par
#' @importFrom graphics plot
#' @importFrom graphics points
#' @importFrom graphics polygon
#' @importFrom grDevices rgb
#' @importFrom stats poly
#' @importFrom stats pnorm
#' @importFrom stats qnorm
#' @importFrom stats na.omit
#' @importFrom rlang .data
#' @import ggplot2
#' @import rdrobust
#'
#'
#' @aliases rdmulti_package
"_PACKAGE"

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rdmulti_package.R
\docType{package}
\name{rdmulti-package}
\alias{rdmulti}
\alias{rdmulti-package}
\alias{rdmulti_package}
\title{Robust Local Polynomial Methods for RD designs with Multiple Cutoffs or Multiple Scores}
\description{
The package \code{rdmulti} implements estimation, inference, and graphical
procedures for Regression Discontinuity (RD) designs with multiple cutoffs or
multiple scores. \code{\link{rdmc}()} provides point estimation and robust
bias-corrected inference for multi-cutoff designs, \code{\link{rdmcplot}()}
provides data-driven RD plots for multi-cutoff designs, and
\code{\link{rdms}()} provides point estimation and robust bias-corrected
inference for multi-score designs. For more details, and related \code{Stata}
and \code{R} packages useful for analysis of RD designs, visit
\url{https://rdpackages.github.io/}.
}
\references{
Calonico, S., M.D. Cattaneo, M. Farrell and R. Titiunik. (2017). \href{https://rdpackages.github.io/references/Calonico-Cattaneo-Farrell-Titiunik_2017_Stata.pdf}{ \code{rdrobust}: Software for Regression Discontinuity Designs}. \emph{Stata Journal} 17(2): 372-404.

Calonico, S., M.D. Cattaneo, and R. Titiunik. (2014). \href{https://rdpackages.github.io/references/Calonico-Cattaneo-Titiunik_2014_Stata.pdf}{Robust Data-Driven Inference in the Regression-Discontinuity Design}. \emph{Stata Journal} 14(4): 909-946.

Calonico, S., M.D. Cattaneo, and R. Titiunik. (2015). \href{https://rdpackages.github.io/references/Calonico-Cattaneo-Titiunik_2015_R.pdf}{ \code{rdrobust}: An R Package for Robust Nonparametric Inference in Regression-Discontinuity Designs}. \emph{R Journal} 7(1): 38-51.

Cattaneo, M.D., L. Keele, R. Titiunik and G. Vazquez-Bare. (2016). \href{https://rdpackages.github.io/references/Cattaneo-Keele-Titiunik-VazquezBare_2016_JOP.pdf}{Interpreting Regression Discontinuity Designs with Multiple Cutoffs}. \emph{Journal of Politics} 78(4): 1229-1248.

Cattaneo, M.D., L. Keele, R. Titiunik and G. Vazquez-Bare. (2020). \href{https://rdpackages.github.io/references/Cattaneo-Keele-Titiunik-VazquezBare_2021_JASA.pdf}{Extrapolating Treatment Effects in Multi-Cutoff Regression Discontinuity Designs}. \emph{Journal of the American Statistical Association} 116(536): 1941, 1952.

Cattaneo, M.D., R. Titiunik and G. Vazquez-Bare. (2020). \href{https://rdpackages.github.io/references/Cattaneo-Titiunik-VazquezBare_2020_Stata.pdf}{Analysis of Regression Discontinuity Designs with Multiple Cutoffs or Multiple Scores}. \emph{Stata Journal} 20(4): 866-891.

Keele, L. and R. Titiunik. (2015). \href{https://rdpackages.github.io/references/Keele-Titiunik_2015_PA.pdf}{Geographic Boundaries as Regression Discontinuities}. \emph{Political Analysis} 23(1): 127-155
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/rdpackages/rdmulti}
  \item Report bugs at \url{https://github.com/rdpackages/rdmulti/issues}
}

}
\author{
Matias D. Cattaneo, Princeton University. \email{matias.d.cattaneo@gmail.com}

Rocio Titiunik, Princeton University. \email{rocio.titiunik@gmail.com}

Gonzalo Vazquez-Bare, UC Santa Barbara. \email{gvazquezbare@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rdmc.R
\name{rdmc}
\alias{rdmc}
\title{Point estimation and robust bias-corrected inference for multi-cutoff designs}
\usage{
rdmc(
  Y,
  X,
  C,
  fuzzy = NULL,
  derivvec = NULL,
  pooled_opt = NULL,
  verbose = FALSE,
  pvec = NULL,
  qvec = NULL,
  hmat = NULL,
  bmat = NULL,
  rhovec = NULL,
  covs_mat = NULL,
  covs_list = NULL,
  covs_dropvec = NULL,
  kernelvec = NULL,
  weightsvec = NULL,
  bwselectvec = NULL,
  scaleparvec = NULL,
  scaleregulvec = NULL,
  masspointsvec = NULL,
  bwcheckvec = NULL,
  bwrestrictvec = NULL,
  stdvarsvec = NULL,
  vcevec = NULL,
  nnmatchvec = NULL,
  cluster = NULL,
  ginv.tolvec = NULL,
  sharpbwvec = NULL,
  level = 95,
  plot = FALSE,
  conventional = FALSE,
  subset = NULL,
  data = NULL
)
}
\arguments{
\item{Y}{outcome variable.}

\item{X}{running variable.}

\item{C}{cutoff variable.}

\item{fuzzy}{specifies a fuzzy design. See \code{rdrobust()} for details.}

\item{derivvec}{vector of cutoff-specific order of derivatives. See
\code{rdrobust()} for details.}

\item{pooled_opt}{options to be passed to \code{rdrobust()} to calculate
pooled estimand.}

\item{verbose}{displays the output from \code{rdrobust} for estimating the
pooled estimand.}

\item{pvec}{vector of cutoff-specific polynomial orders. See
\code{rdrobust()} for details.}

\item{qvec}{vector of cutoff-specific polynomial orders for bias estimation.
See \code{rdrobust()} for details.}

\item{hmat}{matrix of cutoff-specific bandwidths. See \code{rdrobust()} for
details.}

\item{bmat}{matrix of cutoff-specific bandwidths for bias estimation. See
\code{rdrobust()} for details.}

\item{rhovec}{vector of cutoff-specific values of rho. See \code{rdrobust()}
for details.}

\item{covs_mat}{matrix of covariates. See \code{rdrobust()}
for details.}

\item{covs_list}{list of covariates to be used in each cutoff.}

\item{covs_dropvec}{vector indicating whether collinear covariates should be
dropped at each cutoff. See \code{rdrobust()} for details.}

\item{kernelvec}{vector of cutoff-specific kernels. See \code{rdrobust()} for
details.}

\item{weightsvec}{vector of length equal to the number of cutoffs indicating
the names of the variables to be used as weights in each cutoff. See \code{rdrobust()} for details.}

\item{bwselectvec}{vector of cutoff-specific bandwidth selection methods. See
\code{rdrobust()} for details.}

\item{scaleparvec}{vector of cutoff-specific scale parameters. See
\code{rdrobust()} for details.}

\item{scaleregulvec}{vector of cutoff-specific scale regularization
parameters. See \code{rdrobust()} for details.}

\item{masspointsvec}{vector indicating how to handle repeated values at each
cutoff. See \code{rdrobust()} for details.}

\item{bwcheckvec}{vector indicating the value of bwcheck at each cutoff. See
\code{rdrobust()} for details.}

\item{bwrestrictvec}{vector indicating whether computed bandwidths are
restricted to the range or runvar at each cutoff. See \code{rdrobust()} for
details.}

\item{stdvarsvec}{vector indicating whether variables are standardized at
each cutoff. See \code{rdrobust()} for details.}

\item{vcevec}{vector of cutoff-specific variance-covariance estimation
methods. See \code{rdrobust()} for details.}

\item{nnmatchvec}{vector of cutoff-specific nearest neighbors for variance
estimation. See \code{rdrobust()} for details.}

\item{cluster}{cluster ID variable. See \code{rdrobust()} for details.}

\item{ginv.tolvec}{vector of cutoff-specific tolerances for generalized
inverse calculations. See \code{rdrobust()} for details.}

\item{sharpbwvec}{vector indicating whether fuzzy RD bandwidth selection
uses the sharp RD model at each cutoff. See \code{rdrobust()} for details.}

\item{level}{confidence level for confidence intervals. See \code{rdrobust()}
for details.}

\item{plot}{plots cutoff-specific estimates and weights.}

\item{conventional}{reports conventional, instead of robust-bias corrected,
p-values and confidence intervals.}

\item{subset}{optional subset of observations to use.}

\item{data}{optional data frame for resolving string variable names.}
}
\value{
\item{tau}{pooled estimate}
\item{se.rb}{robust bias corrected standard error for pooled estimate}
\item{pv.rb}{robust bias corrected p-value for pooled estimate}
\item{ci.rb.l}{left limit of robust bias corrected CI for pooled estimate}
\item{ci.rb.r}{right limit of robust bias corrected CI for pooled estimate}
\item{hl}{bandwidth to the left of the cutoff for pooled estimate}
\item{hr}{bandwidth to the right of the cutofffor pooled estimate}
\item{Nhl}{sample size within bandwidth to the left of the cutoff for pooled estimate}
\item{Nhr}{sample size within bandwidth to the right of the cutoff for pooled estimate}
\item{B}{vector of bias-corrected estimates}
\item{V}{vector of robust variances of the estimates}
\item{Coefs}{vector of conventional estimates}
\item{W}{vector of weights for each cutoff-specific estimate}
\item{Nh}{vector of sample sizes within bandwidth}
\item{CI}{robust bias-corrected confidence intervals}
\item{H}{matrix of bandwidths}
\item{Pv}{vector of robust p-values}
\item{rdrobust.results}{results from rdrobust for pooled estimate}
\item{cfail}{Cutoffs where rdrobust() encountered problems}
}
\description{
\code{rdmc()} implements point estimation and robust bias-corrected inference
for Regression Discontinuity (RD) designs with multiple cutoffs.
}
\examples{
# Toy dataset
X <- runif(1000,0,100)
C <- c(rep(33,500),rep(66,500))
Y <- (1 + X + (X>=C))*(C==33)+(.5 + .5*X + .8*(X>=C))*(C==66) + rnorm(1000)
# rdmc with standard syntax
tmp <- rdmc(Y,X,C)


}
\references{
Cattaneo, M.D., R. Titiunik and G. Vazquez-Bare. (2020). \href{https://rdpackages.github.io/references/Cattaneo-Titiunik-VazquezBare_2020_Stata.pdf}{Analysis of Regression Discontinuity Designs with Multiple Cutoffs or Multiple Scores}. \emph{Stata Journal}, forthcoming.
}
\author{
Matias D. Cattaneo, Princeton University. \email{matias.d.cattaneo@gmail.com}

Rocio Titiunik, Princeton University. \email{rocio.titiunik@gmail.com}

Gonzalo Vazquez-Bare, UC Santa Barbara. \email{gvazquezbare@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rdms.R
\name{rdms}
\alias{rdms}
\title{Point estimation and robust bias-corrected inference for multi-score designs}
\usage{
rdms(
  Y,
  X,
  C,
  X2 = NULL,
  zvar = NULL,
  C2 = NULL,
  rangemat = NULL,
  xnorm = NULL,
  fuzzy = NULL,
  derivvec = NULL,
  pooled_opt = NULL,
  pvec = NULL,
  qvec = NULL,
  hmat = NULL,
  bmat = NULL,
  rhovec = NULL,
  covs_mat = NULL,
  covs_list = NULL,
  covs_dropvec = NULL,
  kernelvec = NULL,
  weightsvec = NULL,
  bwselectvec = NULL,
  scaleparvec = NULL,
  scaleregulvec = NULL,
  masspointsvec = NULL,
  bwcheckvec = NULL,
  bwrestrictvec = NULL,
  stdvarsvec = NULL,
  vcevec = NULL,
  nnmatchvec = NULL,
  cluster = NULL,
  ginv.tolvec = NULL,
  sharpbwvec = NULL,
  level = 95,
  plot = FALSE,
  conventional = FALSE,
  subset = NULL,
  data = NULL
)
}
\arguments{
\item{Y}{outcome variable.}

\item{X}{running variable.}

\item{C}{vector of cutoffs.}

\item{X2}{if specified, second running variable.}

\item{zvar}{if X2 is specified, treatment indicator.}

\item{C2}{if specified, second vector of cutoffs.}

\item{rangemat}{matrix of cutoff-specific ranges for the running variable.}

\item{xnorm}{normalized running variable to estimate pooled effect.}

\item{fuzzy}{specifies a fuzzy design. See \code{rdrobust()} for details.}

\item{derivvec}{vector of cutoff-specific order of derivatives. See
\code{rdrobust()} for details.}

\item{pooled_opt}{options to be passed to \code{rdrobust()} to calculate
pooled estimand.}

\item{pvec}{vector of cutoff-specific polynomial orders. See
\code{rdrobust()} for details.}

\item{qvec}{vector of cutoff-specific polynomial orders for bias estimation.
See \code{rdrobust()} for details.}

\item{hmat}{matrix of cutoff-specific bandwidths. See \code{rdrobust()} for
details.}

\item{bmat}{matrix of cutoff-specific bandwidths for bias estimation. See
\code{rdrobust()} for details.}

\item{rhovec}{vector of cutoff-specific values of rho. See \code{rdrobust()}
for details.}

\item{covs_mat}{matrix of covariates. See \code{rdplot()} for details.}

\item{covs_list}{list of of covariates to be used in each cutoff.}

\item{covs_dropvec}{vector indicating whether collinear covariates should be
dropped at each cutoff. See \code{rdrobust()} for details.}

\item{kernelvec}{vector of cutoff-specific kernels. See \code{rdrobust()} for
details.}

\item{weightsvec}{vector of length equal to the number of cutoffs indicating
the names of the variables to be used as weights in each cutoff. See \code{rdrobust()} for details.}

\item{bwselectvec}{vector of cutoff-specific bandwidth selection methods. See
\code{rdrobust()} for details.}

\item{scaleparvec}{vector of cutoff-specific scale parameters. See
\code{rdrobust()} for details.}

\item{scaleregulvec}{vector of cutoff-specific scale regularization
parameters. See \code{rdrobust()} for details.}

\item{masspointsvec}{vector indicating how to handle repeated values at each
cutoff. See \code{rdrobust()} for details.}

\item{bwcheckvec}{vector indicating the value of bwcheck at each cutoff. See
\code{rdrobust()} for details.}

\item{bwrestrictvec}{vector indicating whether computed bandwidths are
restricted to the range or runvar at each cutoff. See \code{rdrobust()} for
details.}

\item{stdvarsvec}{vector indicating whether variables are standardized at
each cutoff. See \code{rdrobust()} for details.}

\item{vcevec}{vector of cutoff-specific variance-covariance estimation
methods. See \code{rdrobust()} for details.}

\item{nnmatchvec}{vector of cutoff-specific nearest neighbors for variance
estimation. See \code{rdrobust()} for details.}

\item{cluster}{cluster ID variable. See \code{rdrobust()} for details.}

\item{ginv.tolvec}{vector of cutoff-specific tolerances for generalized
inverse calculations. See \code{rdrobust()} for details.}

\item{sharpbwvec}{vector indicating whether fuzzy RD bandwidth selection
uses the sharp RD model at each cutoff. See \code{rdrobust()} for details.}

\item{level}{confidence level for confidence intervals. See \code{rdrobust()}
for details.}

\item{plot}{plots cutoff-specific and pooled estimates.}

\item{conventional}{reports conventional, instead of robust-bias corrected,
p-values and confidence intervals.}

\item{subset}{optional subset of observations to use.}

\item{data}{optional data frame for resolving string variable names.}
}
\value{
\item{B}{vector of bias-corrected coefficients}
\item{V}{variance-covariance matrix of the estimators} \item{Coefs}{vector of
conventional coefficients} \item{Nh}{vector of sample sizes within bandwidth
at each cutoff} \item{CI}{bias corrected confidence intervals}
\item{H}{bandwidth used at each cutoff} \item{Pv}{vector of robust p-values}
}
\description{
\code{rdms()} implements point estimation and robust bias-corrected inference
for Regression Discontinuity (RD) designs with multiple scores, including
cumulative-cutoff designs and designs with two running variables.
}
\examples{
# Toy dataset: cumulative cutoffs
X <- runif(1000,0,100)
C <- c(33,66)
Y <- (1+X)*(X<C[1])+(0.8+0.8*X)*(X>=C[1]&X<C[2])+(1.2+1.2*X)*(X>=C[2]) + rnorm(1000)
# rmds: basic syntax
tmp <- rdms(Y,X,C)


}
\references{
Cattaneo, M.D., R. Titiunik and G. Vazquez-Bare. (2020). \href{https://rdpackages.github.io/references/Cattaneo-Titiunik-VazquezBare_2020_Stata.pdf}{Analysis of Regression Discontinuity Designs with Multiple Cutoffs or Multiple Scores}. \emph{Stata Journal}, forthcoming.
}
\author{
Matias D. Cattaneo, Princeton University. \email{matias.d.cattaneo@gmail.com}

Rocio Titiunik, Princeton University. \email{rocio.titiunik@gmail.com}

Gonzalo Vazquez-Bare, UC Santa Barbara. \email{gvazquezbare@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rdmcplot.R
\name{rdmcplot}
\alias{rdmcplot}
\title{Data-driven RD plots for multi-cutoff designs}
\usage{
rdmcplot(
  Y,
  X,
  C,
  nbinsmat = NULL,
  binselectvec = NULL,
  scalevec = NULL,
  supportmat = NULL,
  pvec = NULL,
  hmat = NULL,
  kernelvec = NULL,
  weightsvec = NULL,
  covs_mat = NULL,
  covs_list = NULL,
  covs_evalvec = NULL,
  covs_dropvec = NULL,
  ginv.tolvec = NULL,
  masspointsvec = NULL,
  ci = NULL,
  shade = FALSE,
  col_bins = NULL,
  pch_bins = NULL,
  col_poly = NULL,
  lty_poly = NULL,
  col_xline = NULL,
  lty_xline = NULL,
  nobins = FALSE,
  nopoly = FALSE,
  noxline = FALSE,
  nodraw = FALSE,
  subset = NULL,
  data = NULL
)
}
\arguments{
\item{Y}{outcome variable.}

\item{X}{running variable.}

\item{C}{cutoff variable.}

\item{nbinsmat}{matrix of cutoff-specific number of bins. See \code{rdplot()}
for details.}

\item{binselectvec}{vector of cutoff-specific bins selection method. See
\code{rdplot()} for details.}

\item{scalevec}{vector of cutoff-specific scale factors. See \code{rdplot()}
for details.}

\item{supportmat}{matrix of cutoff-specific support conditions. See
\code{rdplot()} for details..}

\item{pvec}{vector of cutoff-specific polynomial orders. See \code{rdplot()}
for details.}

\item{hmat}{matrix of cutoff-specific bandwidths. See \code{rdplot()} for
details.}

\item{kernelvec}{vector of cutoff-specific kernels. See \code{rdplot()} for
details.}

\item{weightsvec}{vector of cutoff-specific weights. See \code{rdplot()} for
details.}

\item{covs_mat}{matrix of covariates. See \code{rdplot()} for
details.}

\item{covs_list}{list of of covariates to be used in each cutoff.}

\item{covs_evalvec}{vector indicating the evaluation point for additional
covariates. See \code{rdrobust()} for details.}

\item{covs_dropvec}{vector indicating whether collinear covariates should be
dropped at each cutoff. See \code{rdrobust()} for details.}

\item{ginv.tolvec}{vector of cutoff-specific tolerances for generalized
inverse calculations. See \code{rdplot()} for details.}

\item{masspointsvec}{vector indicating how to handle repeated values at each
cutoff. See \code{rdplot()} for details.}

\item{ci}{adds confidence intervals of the specified level to the plot. See
\code{rdrobust()} for details.}

\item{shade}{passes the shaded confidence interval option to \code{rdplot()}.}

\item{col_bins}{vector of colors for bins.}

\item{pch_bins}{vector of characters (pch) type for bins.}

\item{col_poly}{vector of colors for polynomial curves.}

\item{lty_poly}{vector of lty for polynomial curves.}

\item{col_xline}{vector of colors for vertical lines.}

\item{lty_xline}{vector of lty for vertical lines.}

\item{nobins}{omits bins plot.}

\item{nopoly}{omits polynomial curve plot.}

\item{noxline}{omits vertical lines indicating the cutoffs.}

\item{nodraw}{omits plot.}

\item{subset}{optional subset of observations to use.}

\item{data}{optional data frame for resolving string variable names.}
}
\value{
\item{clist}{list of cutoffs}
\item{cnum}{number of cutoffs}
\item{X0}{matrix of X values for control units}
\item{X1}{matrix of X values for treated units}
\item{Yhat0}{estimated polynomial for control units}
\item{Yhat1}{estimated polynomial for treated units}
\item{Xmean}{bin average of X values}
\item{Ymean}{bin average for Y values}
\item{CI_l}{lower end of confidence intervals}
\item{CI_r}{upper end of confidence intervals}
\item{cfail}{Cutoffs where rdrobust() encountered problems}
}
\description{
\code{rdmcplot()} implements data-driven Regression Discontinuity (RD) plots
for designs with multiple cutoffs.
}
\examples{
# Toy dataset
X <- runif(1000,0,100)
C <- c(rep(33,500),rep(66,500))
Y <- (1 + X + (X>=C))*(C==33)+(.5 + .5*X + .8*(X>=C))*(C==66) + rnorm(1000)
# rdmcplot with standard syntax
tmp <- rdmcplot(Y,X,C)


}
\references{
Cattaneo, M.D., R. Titiunik and G. Vazquez-Bare. (2020). \href{https://rdpackages.github.io/references/Cattaneo-Titiunik-VazquezBare_2020_Stata.pdf}{Analysis of Regression Discontinuity Designs with Multiple Cutoffs or Multiple Scores}. \emph{Stata Journal}, forthcoming.
}
\author{
Matias D. Cattaneo, Princeton University. \email{matias.d.cattaneo@gmail.com}

Rocio Titiunik, Princeton University. \email{rocio.titiunik@gmail.com}

Gonzalo Vazquez-Bare, UC Santa Barbara. \email{gvazquezbare@gmail.com}
}
